#!/usr/bin/wish

###############################################
# Ren Serral - rserral@ac.upc.es             #
# netmeter.tcl - application launching script #
#       v2.1 (11-2002)                        #
# Roberto Borgione - rborgion@ac.upc.es       #
# netmeter.tcl - application launching script #
#       v2.0 (04-2002)                        #
###############################################

######################################################
### checking version of wish interpretor (8.2 minimum)
######################################################
if { [info tclversion]  < 8.2 } \
{
    tk_messageBox -message "\nTo run this program, wish8.2 or newer required\n"
    exit
}

######################################################
### include source files (procedures,images etc.)
######################################################
source sources/container.tcl 	        ;#container
source sources/setup.tcl 	        ;#BOOT configuration program
source sources/data.tcl 	        ;#images
source sources/config.tcl	        ;#make configuration program data
source sources/main.tcl  	        ;#page 1 Main Setup
source sources/thru4.tcl		;#page 2 Unicast IPv4   procedures
source sources/thru6.tcl		;#page 2 Unicast IPv6   procedures
source sources/thrum4.tcl		;#page 2 Multicast IPv4   procedures
source sources/thrum6.tcl		;#page 2 Multicast IPv6   procedures
source sources/delayu4a.tcl		;#page 3 Unicast IPv4-A procedures
source sources/delayu4b.tcl		;#page 3 Unicast IPv4-B procedures
source sources/delayu6a.tcl		;#page 3 Unicast IPv6-A procedures
source sources/delayu6b.tcl		;#page 3 Unicast IPv6-B procedures
source sources/delaym4a.tcl		;#page 3 Multicast IPv4-A procedures
source sources/delaym4b.tcl		;#page 3 Multicast IPv4-B procedures
source sources/delaym6a.tcl		;#page 3 Multicast IPv6-A procedures
source sources/delaym6b.tcl		;#page 3 Multicast IPv6-B procedures
source sources/help.tcl		        ;#page 4 help procedure
source sources/distribution.tcl	        ;#graphs - distribution
source sources/netgraph.tcl 	        ;#graphs - results (THE ULTIMATE GRAPHTOOL)

############################################################
###                 B  O   O   T
############################################################
BOOT_PROGRAM

##############################################################
###  Program Window Setup
##############################################################
##main window:
wm title     . "Network Performance Meter"
wm iconname  . "Netmeter"
wm protocol  . WM_DELETE_WINDOW Quit
wm resizable . 0 0

option add *Entry.background white
option add *Listbox.background white

##container for program pages (parts)
Container:Crear_container {} 680 720
#and its pages
set page(1) [Container:Obtener_pestanya   1 "Main Options"]
set page(2) [Container:Obtener_pestanya   2 "Throughput"]
set page(3) [Container:Obtener_pestanya   3 "One-way Delay"]
set page(4) [Container:Obtener_pestanya   4 "About"]


############################################################
### Setup Pages Main Frames
############################################################
#page 1: frame left (fL) and right (fR)
set fL [frame $page(1).frameLeft -bd 2 -relief groove]
set fR [frame $page(1).frameRight -bd 2 -relief groove]

#page 2: frames that will show/hide with scenario type changing
set u4tF [frame $page(2).u4throughputFrame]
set u6tF [frame $page(2).u6throughputFrame]
set m4tF [frame $page(2).m4throughputFrame]
set m6tF [frame $page(2).m6throughputFrame]

#call to page 2 procedures to create throughput widgets
Throughput:Tu4	 ;#unicast ipv4 scenario
Throughput:Tu6	 ;#unicast ipv6 scenario just writing yeah..!!!
Throughput:Tm4	 ;#multicast ipv4 scenario
Throughput:Tm6	 ;#multicast ipv6 scenario just writing yeah..!!!

#page 3:
set uni4 [frame $page(3).u4delayFrame]
set uni6 [frame $page(3).u6delayFrame]
set mul4 [frame $page(3).m4delayFrame]
set mul6 [frame $page(3).m6delayFrame]

#call page 3 procedures to create delay widgets
Delay:Du4	;#unicast ipv4 scenario
Delay:Du6	;#unicast ipv6 scenario
Delay:Dm4	;#multicast ipv4 scenario
Delay:Dm6	;#multicast ipv6 scenario

#page 4: help frame
set hF [frame $page(4).helpFrame]
#call page 4 procedure to create help widgets
Help	;# :-)

#frame page that need to be packed ?
pack $u4tF $uni4 $hF -fill both -expand true


############################################################
############################################################
##                 Start PAGE
############################################################
############################################################
MainSetup
#Update the container according to the state variable
Select
#to set up the "newstate" and hide/show the relative frames
#Distribution
#NetGRAPH mcl

