###############################################
# Copyright 2002, 2003 Ren Serral,           #
#                                             #
# delaym4a.tcl - Multicast Delay IPv6 Manager #
#       v1.0 (10-2002)                        #
###############################################

#   This file is part of NetMeter.
#
#   NetMeter is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   NetMeter is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with NetMeter; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

###################################################################          
#------------------------------------------------------------------
#        RUN Multicast DELAY TEST for Ipv6
#------------------------------------------------------------------
#################################################################
proc RunMDelayTest6 {} \
{
   global sender receiver host

   global SGN

   global dlyM        #" dlyM: general ptions for all the defined flows
   global flow6M      #" flow: specific options for the defined flow6M(param,flow_number)
   global mod6M
   global customParamsDM customEndDM
   global runB6M logDelayLB6M
   global mID dID  remoteID command comando
   global stopDrecCmd stopMgenCmd stopDrecPid stopMgenPid
   global success
   global getDrecLog
   global decodepath

   global KeySocket
   global socksize
   global KeySource
   global ipv6SourceAddress
   global _sshCMD


   #check if there is at least one flow6M defined:
   if !$flow6M(number) \
   {
     return
   }
   #set well done test flag to 0:
   set success 0
   #clear status console (message window)
   $runB6M config -text "Wait..."
   $logDelayLB6M delete 0 end
   update idletasks

   #find sender and receiver:
   if $sender(local,ipv6,multicast) {
      ####For the use of the SSH in  IPv4
      set senderHost $host(address,ipv4,multicast)
      set mgenCmd     "$dlyM(mgenPath,ipv6)"
      set gscriptCmd  "cat > /tmp/gscript"
      set stopMgenCmd "killall $SGN mgen6"
   } else {
       ####For the use of the SSH in  IPv4
       set senderHost $sender(address,ipv4,multicast)
       set mgenCmd       "$_sshCMD -l $sender(login,ipv6,multicast) $senderHost $dlyM(mgenPath,ipv6) "
       set gscriptCmd    "$_sshCMD -l $sender(login,ipv6,multicast) $senderHost {cat>/tmp/gscript}"
       set stopMgenCmd   "$_sshCMD -l $sender(login,ipv6,multicast) $senderHost killall $SGN mgen6"
   }


   ##################################################################
   # NOTE over MGEN-DREC SIGINT: 
   # If you don't use SIGINT drec don't finish to write the Received
   # buffer missing the last 5% of output data!!!!
   # also MGEN don't put statistics information :(
   ##################################################################


   if $receiver(local,ipv6) {
      ####For the use of the SSH in  IPv4
      set receiverHost $host(address,ipv4,multicast)
      set drecCmd $dlyM(drecPath,ipv6)
      set dscriptCmd  "cat > /tmp/dscript"
      set stopDrecCmd "killall $SGN drec6"
      set getDrecLog  "$decodepath /tmp/output_file.bin | cat"
   } else {
      ####For the use of the SSH in  IPv4
      set receiverHost $receiver(address,ipv4,multicast)
      set drecCmd     "$_sshCMD -l $receiver(login,ipv6)  $receiverHost $dlyM(drecPath,ipv6) "
      set dscriptCmd  "$_sshCMD -l $receiver(login,ipv6)  $receiverHost {cat >/tmp/dscript}"
      set getDrecLog  "$_sshCMD -l $receiver(login,ipv6)  $receiverHost $decodepath /tmp/output_file.bin | cat"
      set stopDrecCmd "$_sshCMD -l $receiver(login,ipv6)  $receiverHost killall $SGN drec6"
   }


   #mgen interface
   if $sender(local,ipv6,multicast) {
      set sInterface $host(interface,ipv6,multicast)
   } else {
      set sInterface $sender(interface,ipv6,multicast)
   }

   if {[string trim $sInterface] != ""} {
      lappend mgenCmd " -i $sInterface "
   }


   lappend mgenCmd " /tmp/gscript "
   lappend drecCmd " /tmp/dscript  /tmp/output_file.bin"

   $logDelayLB6M insert end "Building scripts..."
   $logDelayLB6M see end
   update idletasks

   lappend gscriptList "# mgen launching script"
   lappend dscriptList "# drec launching script"


   ############################################################
   #tk_messageBox -message "Mgen      command:\n  $mgenCmd"
   #tk_messageBox -message "GSCRIPT   command:\n  $gscriptCmd"
   #tk_messageBox -message "Stop MGen command:\n  $stopMgenCmd"
   #tk_messageBox -message "DSCRIPT   command:\n  $dscriptCmd"
   #tk_messageBox -message "DREC      command:\n  $drecCmd"
   #tk_messageBox -message "STOP_DREC command:\n  $stopDrecCmd"
   #tk_messageBox -message "GETlog    command:\n  $getDrecLog"
   ############################################################


   #if we have a start command
   if $dlyM(startBool) {
       #[trim]->kill space & tab chars
       #if ($dlyM(start)!="") then .....
       if {[string trim $dlyM(start)] != ""} {
          lappend gscriptList "START $dlyM(start)"
          lappend dscriptList "START $dlyM(start)"
       }
   }


   #if we have a srcPort command for mgen
   #the port for the source flow6M is ONE
   #for drec we have to set the ports for all the flows
   if $dlyM(srcPortBool) {
       if {[string trim $dlyM(srcPort)] != ""} {
       lappend gscriptList "PORT $dlyM(srcPort)"
       }
   }
   if $KeySocket {
       if {[string trim  $socksize] != ""} {
       lappend gscriptList "SOCKBUFSIZE $socksize"
       }
   }
   if $KeySource {
       if {[string trim $ipv6SourceAddress] != ""} {
       lappend gscriptList "SOURCE $ipv6SourceAddress"
       }
   }

   #drec interface
   if $receiver(local,ipv6) {
   set rInterface $host(interface,ipv6,multicast)
   } else {
   set rInterface $receiver(interface,ipv6)
   }

   if {[string trim $rInterface] != ""} {
   lappend dscriptList "INTERFACE $rInterface"
   }

   #more flow6M with different destination port
   #ports listened by drec:
   for {set i 1} {$i <= $flow6M(number)} {incr i} {
   lappend portList $flow6M(port,$i)
   }

   #this loop will run at least once,
   #so portList will certainly exist
   lappend dscriptList "PORT [join $portList {,}]"

   ###################################################################
   #tk_messageBox -message "DREC Script List:\n  $dscriptList"
   #tk_messageBox -message "MGEN Script List:\n  $gscriptList"
   ###################################################################
   #Update the OLD use of  SSH in  IPv4
   set receiverHost $receiver(address,ipv6,multicast)


   #deal with flows:
   #construct two temporary lists with commands that
   #will be sorted chronologically afterwards (see the start time)
   lappend drecTempList ""
   for {set i 1} {$i <= $flow6M(number)} {incr i} \
   {
      set unresv 0
      #temporary line
      set tempLine "$flow6M(start,$i) $i ON $receiverHost  $flow6M(port,$i) \
      $flow6M(pattern,$i) $flow6M(rate,$i) $flow6M(size,$i)"
      #the flow6M(i) NO use RSVP parameters 
      #the flow6M(i) NO use TYPE of SERVICE parameters ?
      lappend mgenTempList $tempLine 
      #note: a generic flux(i) is linked with $mod6M(number,$i) modify flows
      #deal with MOD data
      for {set j 1} {$j <= $mod6M(number,$i)} {incr j} \
      {
         set tempLine "$mod6M(start,$j,$i) $i MOD $receiverHost  $flow6M(port,$i) \
           $mod6M(pattern,$j,$i) $mod6M(rate,$j,$i) $mod6M(size,$j,$i)"
         #the mod6M(i)  NO use RSVP parameters 
         #the mod6M(i)  NO use TYPE of SERVICE parameters 

         lappend mgenTempList $tempLine
      }
      lappend mgenTempList "$flow6M(stop,$i) $i OFF"
   }
   #sort mgenTempList and drecTempList
   if [catch {lsort -index 0 -integer $mgenTempList} mgenTempList] \
   {
      tk_messageBox -message "All flows start-stop values\nmust be integers"
      return
   }

   #all time values are integers
   #no need to catch next command (uses the same start-stop values)
   if {[string trim $drecTempList] != ""} \
   {
      set drecTempList [lreplace $drecTempList 0 0]
      set drecTempList [lsort -index 0 -integer $drecTempList]
      set dscriptList  [concat $dscriptList $drecTempList]
   }
   set gscriptList [concat $gscriptList $mgenTempList]
   $logDelayLB6M insert end "Done"
   $logDelayLB6M see end
   update idletasks

   ###################################################################
   #tk_messageBox -message "DREC Script List:\n  $dscriptList"
   #tk_messageBox -message "MGEN Script List:\n  $gscriptList"
   ###################################################################
   #send scripts to their right place:
   $logDelayLB6M insert end "Sending mgen script..."
   $logDelayLB6M see end
   update idletasks

   ###################################################################
   #tk_messageBox -message "send (mgen script)  command:\n  $gscriptCmd"
   #tk_messageBox -message "send (drec script)  command:\n  $dscriptCmd"
   ###################################################################


   if [catch {open "|$gscriptCmd" w} fID] \
   {
      #err message:
      tk_messageBox -message $fID
      $runB6M config -text "Run Test" -command RunMDelayTest6
      return
   }
   #else, transfer data from list to script
   #flush $fID
   ###################################################################
   #tk_messageBox -message " (OPEN) send (mgen script) command executed"
   ###################################################################
   foreach elem $gscriptList \
   {
      #Write from $elem to a channel($fID)
      puts $fID $elem
   }
   #flush $fID
   if [catch {close $fID} err] \
   {
      tk_messageBox -message $err
      $runB6M config -text "Run Test" -command RunMDelayTest6
      return
   }
   ###################################################################
   #tk_messageBox -message " (RUN and CLOSE) send (mgen script) command executed"
   ###################################################################
   $logDelayLB6M insert end "Done"
   $logDelayLB6M insert end "Sending drec script..."
   $logDelayLB6M see end
   update idletasks


   if [catch {open "|$dscriptCmd" w} fID] \
   {
      #err message:
      tk_messageBox -message $fID
      $runB6M config -text "Run Test" -command RunMDelayTest6
      return
   }
   #else, transfer data from list to script
   #flush $fID
   ###################################################################
   #tk_messageBox -message " (OPEN) send (drec script) command executed"
   ###################################################################
   foreach elem $dscriptList \
   {
      #Write from $elem to a channel($fID)
      puts $fID $elem
   }
   #flush $fID
   if [catch {close $fID} err] \
   {
      tk_messageBox -message $err
      $runB6M config -text "Run Test" -command RunMDelayTest6
      return
   }
   $logDelayLB6M insert end "Done"
   $logDelayLB6M see end
   update idletasks
   ###################################################################
   #tk_messageBox -message " (RUN and CLOSE) send (drec script) command executed"
   ###################################################################

   ###################################################################
   #tk_messageBox -message "(mgen) run command:\n  $mgenCmd"
   #tk_messageBox -message "(drec)  run command:\n  $drecCmd"         
   ###################################################################

   #change button:
   $runB6M config -text Stop -command StopMMgen6
   $logDelayLB6M insert end "Starting drec..."
   $logDelayLB6M see end
   update idletasks


   if [catch {open "|$drecCmd" r+ } dID] \
   {
      #error starting drec
      tk_messageBox -message "$dID"
      $runB6M config -text "Run Test" -command RunMDelayTest6
      update idletasks
      return
   }
   #else:
   $logDelayLB6M insert end "Done"
   $logDelayLB6M see end
   update idletasks
   fconfigure $dID -blocking false
   fileevent $dID readable \
   {
     if {[eof $dID]} \
     {
         #to be sure:
         catch {close $dID}
         catch {StopMMgen6}
         #$logDelayLB6M insert end "Drec-pipe closed" 
         $logDelayLB6M insert end "Mgen stopped"
         $logDelayLB6M see end
         update idletasks
         if {$success} \
         {
            $logDelayLB6M insert end "Test completed successfully"
            $logDelayLB6M see end
            update idletasks
            #save the remote files in management station
            SaveMDelayData6
         } else {
            $logDelayLB6M insert end "Test failed"
            $logDelayLB6M see end
            update idletasks
            $runB6M config -text "Run Test" -command RunMDelayTest6
         }
         #$runB6M config -text "Run Test" -command RunDelayTest
         return
      } else {
         gets $dID line
         if {$success!=1} {
            $logDelayLB6M insert end $line
            $logDelayLB6M see end
            update idletasks
         }
      }
   }

   #########################################################################
   #tk_messageBox -message "(mgen)  command ready to execute"
   #########################################################################
   #execute a script when a channel becomes readable (when drec starts
   #to receive data from mgen)

   $logDelayLB6M insert end "Starting mgen..."
   $logDelayLB6M see end
   update idletasks

   # mID = PID of MGEN

   if [catch {open "|$mgenCmd" r+ } mID] \
   {
      #error starting mgen
      tk_messageBox -message $mID
      StopMDrec6
      $runB6M config -text "Run Test" -command RunMDelayTest6
      update idletasks
      return
   }

   #########################################################################
   #tk_messageBox -message "(mgen)  command remote executed" 
   #########################################################################  

   $logDelayLB6M insert end "Done"
   $logDelayLB6M see end
   update idletasks

   set success "0"
   #it works well without this:
   fconfigure $mID -blocking false
   fileevent $mID readable \
   {
      if {[eof $mID]} {
         #mgen is finished:
         if [catch {close $mID} err] {
         tk_messageBox -message $err
         $logDelayLB6M insert end $err
         $logDelayLB6M see end
         }
         catch {StopMDrec6}
         #$logDelayLB6M insert end "Mgen-pipe closed" 
         $logDelayLB6M insert end "Drec stopped"
         $logDelayLB6M see end
         update idletasks
         return
      } else {
         gets $mID line
         if ![string compare "MGEN: Done." $line] \
         {
            set success 1
         }
         $logDelayLB6M insert end $line
         $logDelayLB6M see end
         update idletasks
      }
   }
}



###################################################################
#------------------------------------------------------------------
#                       STOP Multicast DREC
#------------------------------------------------------------------
###################################################################
proc StopMDrec6 {} {
   global runB6M dID logDelayLB6M success 
   global stopDrecCmd

   ##stop drec:

   if [catch {open "|$stopDrecCmd" r+ } err] {
      $logDelayLB6M insert end  $err
      $logDelayLB6M see end
      update idletasks
      return
   }
   #$logDelayLB6M insert end  "$success"
   #$logDelayLB6M see end
   #update idletasks
}



###################################################################
#------------------------------------------------------------------
#                       STOP Multicast MGEN
#------------------------------------------------------------------
###################################################################
proc StopMMgen6 {} \
{
   global mID logDelayLB6M stopMgenCmd


   if [catch {open "|$stopMgenCmd" r+ } err] {
     tk_messageBox -message $err
     $logDelayLB6M insert end  $err
     $logDelayLB6M see end
     update idletasks
   }
}



###################################################################
#------------------------------------------------------------------
#         SAVE Multicast DELAY DATA (save result .DLY and other data)
#    save the files from remote station in management station
#------------------------------------------------------------------
###################################################################

proc SaveMDelayData6 {} \
{
   global success runB6M logDelayLB6M getDrecLog
   global sID scenarioDir dlyM fPath

   set fPath ""

   $runB6M config -text "Wait..." -command {;#do nothing}
   update idletasks

   # box - message
   $logDelayLB6M insert end "Saving data..."
   $logDelayLB6M see end
   update idletasks

   #construct path for local file:

   #the "data" directory:
   set path2 data
   #scenario specific directory:
   set path3 [string trim [string trim $scenarioDir(ipv6,multicast)] {/}]
   #file name:
   set path4 [string trim [string trim $dlyM(resultsFile,ipv6)] {/}]

   if [string compare {} $path3] {
   set fPath "$path2/$path3/$path4"
   } else {
   #no scenario dir specified:
   set fPath "$path2/$path4"
   }


   #see if user has specified a file name
   #if not, save to default.dlyM
   if ![string compare {} $path4] \
   {
      append fPath default.dlyM
      #update path
      set dlyM(resultsFile) default.dlyM
   }

   #append file extension, if needed
   if [string compare {.dlyM} [file extension $fPath]] \
   {
      append fPath .dlyM
   }

   #find the parent directory for this file, to create it
   set dPath [file dirname $fPath]

   #try to create this dir
   if [catch {file mkdir $dPath} dErr] \
   {
      #error creating directory (if it already exists, no error)
      $logDelayLB6M insert end "Failed creating directory structure"
      $logDelayLB6M see end
      $runB6M config -text "Run Test" -command RunMDelayTest6
      update idletasks
      return
   }

   #########################################################################
   #tk_messageBox -message "dPath : $dPath "
   #tk_messageBox -message "fpath : $fPath "
   #tk_messageBox -message "getDrecLog : $getDrecLog "
   #########################################################################

   #check if the file already exists:
   if [file exists $fPath] \
   {
      set answer [tk_messageBox -message "File already exists: Overwrite?" \
      -type yesno -icon question]
      if [string compare $answer yes] {
      #user says no:
      $logDelayLB6M delete 0 end
      $logDelayLB6M insert end "Change results file name"
      $logDelayLB6M insert end "and press Save Data button"
      $logDelayLB6M see end
      $runB6M config -text "Save Data" -command SaveMDelayData6
      update idletasks
      return
   }
   #else, it's ok to continue
   }
   #try to open file in write mode:
   if [catch {open $fPath w} iD] \
   {
      #error opening file
      $logDelayLB6M insert end "Failed opening local file"
      $logDelayLB6M see end
      $runB6M config -text "Run Test" -command RunMDelayTest6
      update idletasks
      return
   }
   #else, we can write.
   #open pipe to read file:


   if [catch {open "|$getDrecLog" r+ } sID] \
   {
      #error opening file
      $logDelayLB6M insert end "Failed opening remote data file"
      $logDelayLB6M see end
      $runB6M config -text "Run Test" -command RunMDelayTest6
      update idletasks
      return
   }

   #else:
   #The fcopy command copies data from one I/O channel,
   #inchan to another I/O channel  in background;
   #if an error occur command is executed
   #sID=socket ID
   #fcopy return nbytes and in case error
   #this is the default implementation of the fcopy (see help)
   fcopy $sID $iD -command [list FinishedM6 $sID $iD]

   #a test showing that procedure is non-bloking:
   #$logDelayLB6M insert end "End of SaveMDelayData6"
   #$logDelayLB6M see end
}



###################################################################
#------------------------------------------------------------------
#         when we finish the remote copy to local file
#------------------------------------------------------------------
###################################################################
proc FinishedM6 {in out bytes {error {}} } \
{
   global logDelayLB6M runB6M

   catch {close $in}
   catch {close $out}

   $logDelayLB6M insert end "$bytes bytes transferred"
   $logDelayLB6M see end
   if [string length $error] {
      $logDelayLB6M insert end $error
      $logDelayLB6M see end
      $runB6M config -text "Run Test" -command RunMDelayTest6
      update idletasks
   } else {
      #at last:
      $logDelayLB6M insert end "Done"
      $logDelayLB6M see end
      ProcessMData6
   }
}



###################################################################
#------------------------------------------------------------------
#   process the local Multicast data file (.DLY) 
#------------------------------------------------------------------
###################################################################
proc ProcessMData6 {} \
{
   global runB6M logDelayLB6M fPath mcID
   global mcalcpath
   global idfile2
 
   catch {unset outList}
   #announce the beginning
   $logDelayLB6M insert end "Processing data..."
   $logDelayLB6M see end
   update idletasks

   set McalcCmd "$mcalcpath <$fPath"
   #tk_messageBox -message "MCALC COMMAND1: \n $McalcCmd"
   #eval exec $McalcCmd
   #eval exec $McalcCmd 
   #set McalcCmd "cat  temp/mcalc.txt"
   #tk_messageBox -message "MCALC COMMAND2: \n $McalcCmd"

   if [catch { open "|$McalcCmd"} mcID] \
   {
      $logDelayLB6M insert end "Processing failed"
      $logDelayLB6M see end
      $runB6M config -text "Run Test" -command RunMDelayTest6
      update idletasks
      return
   }
   set outList ""
   #else, read output in memory:
   fileevent $mcID readable \
   {
      if [eof $mcID] \
      {
         #when done:
#         catch {close $idfile2}
         catch {close $mcID}
         SaveMMcalcData6 $outList
         catch {unset outList}
      } else {
         gets $mcID line
#        puts $idfile2 $line
         lappend outList $line
         ##############################################
         # MCALC OUTPUT - preview from pipe           # 
         # it is not necesary really this step ....   # 
         ##############################################
         #$logDelayLB6M insert end "PIPE READING: $line"
         #$logDelayLB6M see end
         ##############################################
      }
   };
};



###################################################################
#------------------------------------------------------------------
#                       SAVE Multicast MCALC DATA  (.MCL)
#------------------------------------------------------------------
###################################################################
proc SaveMMcalcData6 { outList } \
{
   global runB6M logDelayLB6M flow6M dlyM mod6M
   global fPath
   global xID
   global paramU6 pU6 countPU6 


   #update flow6M data
   for {set i 1} {$i <= $flow6M(number)} {incr i} {
     set flow6M(active,$i) no
     set flow6M(mod,$i) $mod6M(number,$i)
   }

   #show mcalc results:
   eval $logDelayLB6M insert end $outList

   set ok 0
   set lineCount 0

	foreach line $outList {
		if {[scan $line "%s" isFlow] != -1} {
			#tk_messageBox -message "low  \n $line"
			if ![string compare  [string range $line 0 3]  "FLOW"] {
				#a line announcing a flow
				incr ok
				#retrieve flow data
				set count $lineCount

				#flow number, source and destination:
				#scan [lindex $outList $count] "FLOW>%d   SOURCE>%s DESTINATION:%s"   fNr src dst
				scan [lindex $outList $count] "FLOW>%d   SOURCE>%s  DESTINATION>%s"   fNr src dst

				#tk_messageBox -message "FLOW\n $fNr"
				#tk_messageBox -message "source\n $src"
				#k_messageBox -message "source\n $dst"

			#   set fNr2 [string range $line 4 12]
			#   set fNr 1
			#   scan fNr2 ">%d  "  fNr
			#   tk_messageBox -message "Numeromore  \n $fNr"
			#   set src [string range $line 8 [string length $line] ]
			#   set dst [string range $line 8 [string length $line] ]

				set mcalc($fNr,source)       $src
				set mcalc($fNr,destination)  $dst

				#num of packets received
				incr count

				scan [lindex $outList $count] "   Num pkts recvd  : %s" var
				set mcalc($fNr,recvPacketsNumber) $var

				#joined delay:
				incr count
				scan [lindex $outList $count] "   Join delay      : %s" var
				set mcalc($fNr,joinDelay) $var

				#received packet rate:
				incr count
				scan [lindex $outList $count] "   Recv pkt rate   : %s" var
				set mcalc($fNr,recvPacketsRate) $var

				#recv data rate
				incr count
				scan [lindex $outList $count] "   Recv data rate  : %s" var
				set mcalc($fNr,recvDataRate) $var

				#packets dropped:
				incr count
				scan [lindex $outList $count] "   Pkts dropped    : %s" var
				set mcalc($fNr,packetsDropped) $var

				#average delay
				incr count
				scan [lindex $outList $count] "   Ave. Tx Delay   : %s" var
				set mcalc($fNr,averageDelay) $var

				#max delay
				incr count
				scan [lindex $outList $count] "   Max. Tx Delay   : %s" var
				set mcalc($fNr,maxDelay) $var

				#min delay
				incr count
				scan [lindex $outList $count] "   Min. Tx Delay   : %s" var
				set mcalc($fNr,minDelay) $var

				#delay variation:
				incr count
				scan [lindex $outList $count] "   Delay variation : %s" var
				set mcalc($fNr,delayVariation) $var

				#update flow array:
				set flow6M(active,$fNr) yes
			};#end if FLOW:
			incr lineCount
		}
	};#end foreach

   #####################################################################
   # Finally Save the DATA file
   # set file name: the same as fPath, but different extension:.mcl file
   #####################################################################

   set mPath [file rootname $fPath]
   append mPath .mcl

   #open file
   if [catch {open $mPath w} xID] {
   $logDelayLB6M insert end "Failed saving results"
   $logDelayLB6M see end
   $runB6M config -text "Run Test" -command RunMDelayTest6
   update idletasks
   return
   }
   #else, start writing data:
   for {set i 1} {$i <= $flow6M(number)} {incr i} \
   {
      puts $xID " "
      puts $xID "Mgen measure"
      puts $xID "Flow name: $dlyM(title,ipv6) $flow6M(name,$i)"
      puts $xID " "
      puts $xID "Flow IPv6 ID $i"
      puts $xID "-Flow Relative Start (msec):  $flow6M(start,$i)"
      puts $xID "-Flow Relative Stop  (msec):   $flow6M(stop,$i)"
      puts $xID "Pattern  $flow6M(pattern,$i)"
      puts $xID "-Send Rate (packets/sec): $flow6M(rate,$i)"
      puts $xID "-UDP Packet Size (bytes): $flow6M(size,$i)"
      puts $xID "-IP Packet Size (bytes): [expr $flow6M(size,$i) +48]"
      puts $xID "Nr. of MODs   $flow6M(mod,$i)"
      if $dlyM(startBool) {
         puts $xID "Mgen Absolute START   $dlyM(start)"
      }
      #tk_messageBox -message "flag: $flow6M(active,$i)"
      #####################################################
      # MCALC data
      #####################################################
      puts $xID "Flow Source        $mcalc($i,source)"
      puts $xID "Flow Destination   $mcalc($i,destination)"
      puts $xID "-Received packets: $mcalc($i,recvPacketsNumber)"
      puts $xID "-Join delay (sec): $mcalc($i,joinDelay)"
      puts $xID "-Recv rate (packets/sec): $mcalc($i,recvPacketsRate)"
      puts $xID "-Recv data rate (kbps): $mcalc($i,recvDataRate)"
      puts $xID "-Packets dropped: $mcalc($i,packetsDropped)"
      puts $xID "-Average delay(sec):   $mcalc($i,averageDelay)"
      puts $xID "-Max delay (sec): $mcalc($i,maxDelay)"
      puts $xID "-Min delay (sec): $mcalc($i,minDelay)"
      puts $xID "-Delay variation (sec): $mcalc($i,delayVariation)"
      #custom params:
      set first 1
      set selList [lsort [array names paramU6]]
      for {set count 0} {$count < $countPU6} {incr count} \
      {
         set tmp [lindex $selList $count]
         if [string compare "(parameters)" $tmp] \
         { 
            if { $first == 1 } { puts $xID "Custom parameters:" }
            puts $xID "-$tmp" 
            set first 0 
         }
      }

   }
   catch {close $xID}
   $logDelayLB6M insert end "Done "
   $logDelayLB6M see end
   $runB6M config -text "Run Test" -command RunMDelayTest6
   update idletasks
}



