###############################################
# Copyright 2002, 2003 Ren Serral,           #
#                      Roberto Borgione       #
# setup.tcl - Set-Up loading and saving       #
#       v2.0 (04-2002)                        #
#       v2.01(02-2003)                        #
###############################################

#   This file is part of NetMeter.
#
#   NetMeter is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   NetMeter is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with NetMeter; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

###################################################################
#------------------------------------------------------------------
#                   BOOT PROCEDURE
#------------------------------------------------------------------
###################################################################
proc BOOT_PROGRAM {} \
{
   global _distrCMD
   global _graphCMD
   global _xviewCMD
   global _correlCMD
   global _sshCMD
   global _defaultCFG
   global _filedatCFG
   global FIRST_RUN
   global SGN
   global MCALC_input

   set _defaultCFG "conf/netmeter.cfg"
   set _filedatCFG "bin/config.dat"
   set _distrCMD   "bin/distr "
   set _graphCMD   "bin/graph2002 -cfg"
   set _sshCMD     "ssh -x "
   set _correlCMD  "bin/correlation "

   ######################################
   #This is for FREEBSD-KDE platform
   ######################################
   set _xviewCMD   "gqview "
   set SGN "-SIGINT"
   set MCALC_input "<"

   ######################################
   #This is for LINUX-GNOME platform 
   ######################################
   set _xviewCMD   "ee "
   set SGN "-INT"
   set MCALC_input "<"
   
   set FIRST_RUN 0
   ######################################
   #Loading configuration on sources/config.dat
   #Try to load the file with the name of the last configuration used
   ######################################
   if [catch {open $_filedatCFG r} idfile2] {
      #Load Default Configuration file
      load_configuration_file $_defaultCFG
   } else {
      gets $idfile2 line
      catch {close $idfile2}
      if [catch {open $line r} idfile2] \
      {
         #Load Default Configuration file
         load_configuration_file $_defaultCFG
      } else {
         load_configuration_file $line
      }
   }

   ######################################
   ## verify the directory structure
   ######################################
   if [catch {file mkdir temp} dErr] \
   {
      tk_messageBox -message "temp directory not found "
      return
   }
   if [catch {file mkdir graphs} dErr] \
   {
      tk_messageBox -message "graphs directory not found "
      return
   }
   ######################################
   ## Setup all global variable not in config
   ######################################
   set FIRST_RUN 1
}
